@@ -49,8 +49,8 @@ have to install it manually (at least one user reported he had to).
 Unstable (development) version
 ------------------------------
 
-Just run ``hg clone http://dev.pitrou.net:8000/ hgsvn`` and you'll get the
-contents of the development repository.
+Just run ``hg clone http://hg.pitrou.net/public/hgsvn/main hgsvn`` and you'll
+get the contents of the development repository.
 
 --------
 Features
@@ -144,6 +144,58 @@ changes to SVN automatically.
 History
 -------
 
+hgsvn 0.1.6
+-----------
+
+Improvements:
+
+* Prefix commit messages with [svn r123] (where 123 is the corresponding SVN
+  revision number), rather than just [svn]. Also, trim leading whitespace in
+  the original commit message, to avoid blank changeset summaries when the
+  message begins with a carriage return.
+
+* Introduce a .hgsvn private directory at the top of the working copy. This
+  will us to store various things in a common location without polluting the
+  base directory.
+
+* Introduce a lock file (named .hgsvn/lock) to disallow running two hgpullsvn
+  instances in parallel on the same repository. The locking mechanism is
+  imported from mercurial's own mercurial.lock. If the mercurial package is
+  not available, a dummy lock is used instead. Initial patch by Ori Peleg.
+
+* Add a --no-hgignore option to hgimportsvn, for situations where the source
+  SVN repository already contains a versioned .hgignore file. Patch by
+  Ori Peleg.
+
+* hgsvn can now be bundled as standalone executables using py2exe. Patch by
+  Paul Moore.
+
+* More descriptive error message when either hg or svn cannot be executed
+  (e.g. not installed). Patch by Joonas Paalasmaa.
+
+Bug fixes:
+
+* Very long commit messages (> 16000 characters) are provided to Mercurial
+  through a temporary file rather than the command line, which could fail
+  on some platforms. The corresponding commit messages were generated by
+  svnmerge. Reported by Ralf Schmitt.
+
+* Filenames starting with a hyphen were considered by hg and SVN as
+  command-line options. Report and patch by Mirko Friedenhagen.
+
+* If the last hg changeset mapped to more than one SVN revision, trying to
+  update again with ``hgpullsvn`` failed.
+
+* A replaced directory can have added and removed files without them being
+  mentioned in the SVN log; we must detect those files ourselves.
+
+* More robust atomicity check for (hg commit, hg tag) sequence. Reported by
+  Florent Guillaume.
+
+* Fix a bug when comparing local filesystem paths on Windows. We now invoke
+  os.path.normcase on both paths before comparing. Reported by Pavol Murin.
+
+
 hgsvn 0.1.5
 -----------
 
@@ -259,7 +311,7 @@ __all__ = []
 
 __author__ = 'Antoine Pitrou'
 __license__ = 'GNU General Public License (version 3 or later)'
-__versioninfo__ = (0, 1, 5)
+__versioninfo__ = (0, 1, 6)
 
 base_version = '.'.join(map(str, __versioninfo__))
 full_version = base_version
