@@ -8,22 +8,43 @@ import time
 from subprocess import Popen, PIPE, STDOUT
 import shutil
 import stat
+import sys
+import traceback
 
 try:
     import commands
 except ImportError:
     commands = None
 
+# We import the lock logic from Mercurial if it is available, and fall back
+# to a dummy (always successful) lock if not.
+try:
+    from mercurial.lock import lock as _lock, LockHeld
+except ImportError:
+    class _StubLock(object):
+        def __init__(self, file, timeout=None):
+            pass
+        def lock(self):
+            pass
+        def release(self):
+            pass
+    _lock = _StubLock
+    class LockHeld(Exception):
+        pass
+
+
+hgsvn_private_dir = ".hgsvn"
+hgsvn_lock_file = "lock"
+
 if "SVN_ASP_DOT_NET_HACK" in os.environ:
     svn_private_dir = "_svn"
 else:
     svn_private_dir = '.svn'
 
-hg_commit_prefix = "[svn] "
+hg_commit_prefix = "[svn r%d] "
 # This seems sufficient for excluding all .svn (sub)directories
 hg_exclude_options = ["-X", svn_private_dir, "-X", "**/%s" % svn_private_dir]
 
-
 # Windows compatibility code by Bill Baxter
 if os.name == "nt":
     def find_program(name):
@@ -88,7 +109,13 @@ def shell_quote(s):
 def _run_raw_command(cmd, args, fail_if_stderr=False):
     cmd_string = "%s %s" % (cmd,  " ".join(map(shell_quote, args)))
     print "*", cmd_string
-    pipe = Popen([cmd] + args, executable=cmd, stdout=PIPE, stderr=PIPE)
+    try:
+        pipe = Popen([cmd] + args, executable=cmd, stdout=PIPE, stderr=PIPE)
+    except OSError:
+        etype, value = sys.exc_info()[:2]
+        raise ExternalCommandFailed(
+            "Failed running external program: %s\nError: %s"
+            % (cmd_string, "".join(traceback.format_exception_only(etype, value))))
     out, err = pipe.communicate()
     if pipe.returncode != 0 or (fail_if_stderr and err.strip()):
         raise ExternalCommandFailed(
@@ -120,6 +147,10 @@ def run_command(cmd, args=None, bulk_args=None, encoding=None, fail_if_stderr=Fa
     cmd = find_program(cmd)
     if not bulk_args:
         return _run_raw_command(cmd, map(_transform_arg, args), fail_if_stderr)
+    # If one of bulk_args starts with a slash (e.g. '-foo.php'),
+    # hg and svn will take this as an option. Adding '--' ends the search for
+    # further options.
+    args.append("--")
     max_args_num = 254
     i = 0
     out = ""
@@ -190,19 +221,44 @@ def skip_dirs(paths, basedir="."):
 
 def hg_commit_from_svn_log_entry(entry, files=None):
     """
-    Given an SVN log entry and an optional sequence of files, do an hg commit.
+    Given an SVN log entry and an optional sequence of files, turn it into
+    an appropriate hg changeset on top of the current branch.
     """
     # This will use the local timezone for displaying commit times
     timestamp = int(entry['date'])
     hg_date = str(datetime.fromtimestamp(timestamp))
     # Uncomment this one one if you prefer UTC commit times
     #hg_date = "%d 0" % timestamp
-    options = ["ci", "-m", hg_commit_prefix + entry['message'],
-        "-u", entry['author'], "-d", hg_date]
-    if files:
-        options += list(files)
-    run_hg(options)
-    hg_tag_svn_rev(entry['revision'])
+    message = (hg_commit_prefix % entry['revision']) + entry['message'].lstrip()
+    commit_file = None
+    try:
+        # Very long commit messages must not be passed on the command line,
+        # or command invokation can fail on some platforms.
+        if len(message) > 16000:
+            commit_file = os.path.join(hgsvn_private_dir,
+                "commit-%r.txt" % entry['revision'])
+            f = open(commit_file, "wb")
+            f.write(message)
+            f.close()
+            msg_options = ["-l", commit_file]
+        else:
+            msg_options = ["-m", message]
+        options = ["ci"] + msg_options + ["-u", entry['author'], "-d", hg_date]
+        if files:
+            run_hg(options, files)
+        else:
+            run_hg(options)
+    finally:
+        if commit_file and os.path.exists(commit_file):
+            os.remove(commit_file)
+    try:
+        hg_tag_svn_rev(entry['revision'])
+    except:
+        # Rollback the transaction.
+        last_rev = get_svn_rev_from_hg()
+        if last_rev != entry['revision']:
+            run_hg(["rollback"])
+        raise
 
 def hg_tag_svn_rev(rev_number):
     """
@@ -216,8 +272,26 @@ def get_svn_rev_from_hg():
     or None if no match found.
     """
     tags = run_hg(['parents', '--template', '{tags}']).strip().split()
-    for tag in tags:
-        if tag.startswith('svn.'):
-            return int(tag.split('.')[1])
+    revs = [int(tag[4:]) for tag in tags if tag.startswith('svn.')]
+    # An hg changeset can map onto several SVN revisions, for example if a
+    # revision only changed SVN properties.
+    if revs:
+        return max(revs)
     return None
 
+def fixup_hgsvn_dir(basedir='.'):
+    """
+    Create the hgsvn directory if it does not exist. Useful when using
+    repos created by a previous version.
+    """
+    target = os.path.join(basedir, hgsvn_private_dir)
+    if not os.path.exists(target):
+        os.mkdir(target)
+
+def get_hgsvn_lock(basedir='.'):
+    """
+    Get a lock using the hgsvn lock file.
+    """
+    return _lock(os.path.join(basedir, hgsvn_private_dir, hgsvn_lock_file),
+        timeout=0)
+
