@@ -2,6 +2,7 @@
 from hgsvn import base_version, full_version
 
 import sys
+import os
 from optparse import SUPPRESS_HELP
 
 
@@ -40,12 +41,13 @@ def run_parser(parser, long_help=""):
         parser.print_help()
         sys.exit(0)
     if options.show_version:
+        prog_name = os.path.basename(sys.argv[0])
         if options.help2man:
-            print "hgsvn", base_version
+            print prog_name, base_version
             print
             print copyright_message
         else:
-            print "hgsvn", full_version
+            print prog_name, full_version
         sys.exit(0)
     return options, args
 
