@@ -10,11 +10,11 @@ SVN URL (for example 'trunk').
 from hgsvn.common import (
     run_hg, run_svn, skip_dirs, rmtree,
     hg_commit_from_svn_log_entry, hg_exclude_options,
-    svn_private_dir,
+    svn_private_dir, hgsvn_private_dir, fixup_hgsvn_dir,
 )
 from hgsvn.svnclient import (
     get_first_svn_log_entry, get_last_svn_log_entry,
-    get_svn_info, get_svn_status, svn_checkout,
+    get_svn_info, get_svn_status, svn_checkout, get_svn_versioned_files,
 )
 from hgsvn.run.common import run_parser, display_parser_error
 
@@ -36,6 +36,8 @@ def main():
         help="SVN revision to checkout from")
     parser.add_option("-p", "--svn-peg", type="int", dest="svn_peg",
         help="SVN peg revision to locate checkout URL")
+    parser.add_option("--no-hgignore", dest="hgignore", action="store_false",
+        default=True, help="Don't autogenerate .hgignore")
     (options, args) = run_parser(parser, __doc__)
     if not 1 <= len(args) <= 2:
         display_parser_error(parser, "incorrect number of arguments")
@@ -67,6 +69,7 @@ def main():
             raise ValueError("%s is not a directory" % local_repo)
     else:
         os.mkdir(local_repo)
+    fixup_hgsvn_dir(local_repo)
     os.chdir(local_repo)
 
     # Get log entry for the SVN revision we will check out
@@ -127,17 +130,19 @@ def main():
     svn_ignores = filter(None, [s.strip()
         for s in run_svn(['propget', 'svn:ignore', svn_url]).splitlines()])
 
-    # Generate .hgignore file to ignore .svn directories
-    f = open(".hgignore", "a")
-    try:
-        f.write("\n# Automatically generated by `hgimportsvn`\n")
-        f.write("syntax:glob\n%s\n" % svn_private_dir)
-        f.write("\n# These lines are suggested according to the svn:ignore property")
-        f.write("\n# Feel free to enable them by uncommenting them")
-        f.write("\nsyntax:glob\n")
-        f.write("".join("#%s\n" % s for s in svn_ignores))
-    finally:
-        f.close()
+    if options.hgignore:
+        # Generate .hgignore file to ignore .svn and .hgsvn directories
+        f = open(".hgignore", "a")
+        try:
+            f.write("\n# Automatically generated by `hgimportsvn`\n")
+            f.write("syntax:glob\n%s\n" %
+                "\n".join([svn_private_dir, hgsvn_private_dir]))
+            f.write("\n# These lines are suggested according to the svn:ignore property")
+            f.write("\n# Feel free to enable them by uncommenting them")
+            f.write("\nsyntax:glob\n")
+            f.write("".join("#%s\n" % s for s in svn_ignores))
+        finally:
+            f.close()
 
     # Stay on the same filesystem so as to have fast moves
     checkout_dir = tempfile.mkdtemp(dir=".")
@@ -145,12 +150,8 @@ def main():
     try:
         # Get SVN manifest and checkout
         svn_checkout(svn_url, checkout_dir, svn_rev)
-        #svn_client.checkout(svn_url, checkout_dir, revision=svn_rev)
-        svn_manifest = []
+        svn_manifest = get_svn_versioned_files(checkout_dir)
 
-        for e in get_svn_status(checkout_dir):
-            if e['path'] and e['type'] == 'normal':
-                svn_manifest.append(e['path'])
         svn_files = set(skip_dirs(svn_manifest, checkout_dir))
         svn_dirs = sorted(set(svn_manifest) - svn_files)
         svn_files = list(svn_files)
