@@ -7,10 +7,11 @@ Mercurial changesets.
 from hgsvn.common import (
     run_command, run_hg, run_svn, skip_dirs,
     hg_commit_from_svn_log_entry, hg_tag_svn_rev, hg_exclude_options,
-    hg_commit_prefix, get_svn_rev_from_hg,
+    hg_commit_prefix, get_svn_rev_from_hg, get_hgsvn_lock, LockHeld,
+    fixup_hgsvn_dir,
 )
 from hgsvn.svnclient import (
-    get_svn_info,
+    get_svn_info, get_svn_versioned_files,
     iter_svn_log_entries,
 )
 from hgsvn.errors import (
@@ -65,6 +66,7 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
     removed_paths = []
     changed_paths = []
     unrelated_paths = []
+    replaced_paths = {}
     for d in log_entry['changed_paths']:
         # e.g. u'/branches/xmpp-subprotocols-2178-2/twisted/words/test/test_jabberxmlstream.py'
         p = d['path']
@@ -96,7 +98,7 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
                 else:
                     # Extract actual copied files (hg doesn't track dirs
                     # and will refuse "hg copy -A" with dirs)
-                    r = run_hg(["st", "-nc", old_p])
+                    r = run_hg(["st", "-nc"], [old_p])
                     for old_f in r.splitlines():
                         f = p + old_f[len(old_p):]
                         copied_paths.append((old_f, f))
@@ -107,11 +109,29 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
             added_paths.append(p)
         elif d['action'] == 'D':
             removed_paths.append(p)
+        elif d['action'] == 'R':
+            # (R)eplaced directories can have added and removed files without
+            # them being mentioned in the SVN log => we must detect those files
+            # ourselves.
+            # (http://svn.python.org/projects/python/branches/py3k, rev 59625)
+            if os.path.isdir(p):
+                replaced_paths[p] = get_svn_versioned_files(
+                    os.path.join(svn_wc, p))
+            else:
+                # We never know what twisty semantics (R) can have. addremove
+                # is safest.
+                added_paths.append(p)
 
     # Update SVN + add/remove/commit hg
     try:
         if changed_paths:
             run_svn(["up", "--ignore-externals", "-r", svn_rev, svn_wc])
+            for p, old_contents in replaced_paths.items():
+                old_contents = set(old_contents)
+                new_contents = set(get_svn_versioned_files(
+                    os.path.join(svn_wc, p)))
+                added_paths.extend(p + '/' + f for f in new_contents - old_contents)
+                removed_paths.extend(p + '/' + f for f in old_contents - new_contents)
             if added_paths:
                 # Use 'addremove' because an added SVN directory may
                 # overwrite a previous directory with the same name.
@@ -119,15 +139,15 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
                 run_hg(["addremove"] + hg_exclude_options, added_paths)
             for old, new in copied_paths:
                 try:
-                    run_hg(["copy", "-A", old, new])
+                    run_hg(["copy", "-A"], [old, new])
                 except ExternalCommandFailed:
                     # Maybe the "old" path is obsolete, i.e. it comes from an
                     # old SVN revision and was later removed.
-                    s = run_hg(['st', '-nd', old])
+                    s = run_hg(['st', '-nd'], [old])
                     if s.strip():
                         # The old path is known by hg, something else happened.
                         raise
-                    run_hg(["add", new])
+                    run_hg(["add"], [new])
             if removed_paths:
                 run_hg(["remove", "-A"], removed_paths)
             hg_commit_from_svn_log_entry(log_entry)
@@ -141,12 +161,6 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
         time.sleep(0.3)
         run_svn(["cleanup"])
         hgsvn_rev = get_svn_rev_from_hg()
-        if hgsvn_rev is None:
-            if run_hg(['parents', '--template', '{desc}']).startswith(hg_commit_prefix):
-                # hg commit did occur but it's not sure everything is ok.
-                # (sometimes hg commit interrupted at the very end leaves the WC
-                # in a corrupt state).
-                run_hg(["rollback"])
         if hgsvn_rev != svn_rev:
             # Unless the tag is there, revert to the last stable state
             run_svn(["up", "--ignore-externals", "-r", current_rev, svn_wc])
@@ -156,7 +170,7 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
     return svn_rev
 
 
-def main():
+def locked_main():
     usage = "usage: %prog [-p SVN peg rev] [--help]"
     parser = OptionParser(usage)
     parser.add_option("-p", "--svn-peg", type="int", dest="svn_peg",
@@ -245,6 +259,19 @@ def main():
         traceback.print_exc()
 
 
+def main():
+    # We must do this before trying to take the lock file
+    fixup_hgsvn_dir('.')
+    try:
+        l = get_hgsvn_lock()
+    except LockHeld, e:
+        print "Aborting:", e
+        return 1
+    try:
+        return locked_main()
+    finally:
+        l.release()
+
+
 if __name__ == "__main__":
     sys.exit(main() or 0)
-
